from typing import Union
from selenium.webdriver.remote.webelement import WebElement
from selenium.common.exceptions import NoSuchElementException


class ListProduct:

    # Nesne içindeki ad elemanı seçicisi
    name_locator = ()
    # Nesne içindeki link elemanı seçicisi
    link_locator = ()
    # Nesne içindeki ürün kodu elemanı seçicisi
    code_locator = ()
    # Nesne içindeki indirim beliteci elemanları seçicileri
    # İndirimli fiyat (üstü çizili) veya çok al az öde gibi
    # elemanların seçicileri "tuple" şeklinde eklenir
    discount_locators = []

    def __init__(self, element: WebElement) -> None:
        self.element = element
        self.name = None
        self.link = None
        self.code = None

        if self.name_locator:
            self.name = self.get_name()

        if self.link_locator:
            self.link = self.get_link()

        if self.code_locator:
            self.code = self.get_code()

    def get_name(self) -> Union[str, None]:
        try:
            el = self.element.find_element(*self.name_locator)
            return el.text.strip()
        except NoSuchElementException:
            return None

    def get_link(self) -> Union[str, None]:
        try:
            el = self.element.find_element(*self.link_locator)
            return el.get_attribute("href")
        except NoSuchElementException:
            return None

    def get_code(self) -> Union[str, None]:
        try:
            el = self.element.find_element(*self.code_locator)
            return el.text.strip()
        except NoSuchElementException:
            return None

    def is_discount(self) -> bool:
        for locator in self.discount_locators:
            try:
                el = self.element.find_element(*locator)
                if el:
                    return True
            except NoSuchElementException:
                pass

        return False



import locale
from .category_page import CategoryPage
from .product_page import ProductPage
from ..filesytem import File
from ..date import Date
from ..db import DB
from .category_page import CategoryPage
from .product_page import ProductPage
from typing import Dict, List, Self, Tuple, Union
from selenium_web_browser.browser import Browser
from selenium.common.exceptions import TimeoutException
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.chrome.webdriver import WebDriver
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.support import expected_conditions as ec


class Market:

    URL: str = ""
    IMAGES_DIR: str = ""

    browser: Union[Browser, None] = None
    driver: Union[WebDriver, None] = None
    wait: Union[WebDriverWait, None] = None
    logger = None

    # Veritabanı özellikleri
    db_path = ""
    db: Union[DB, None] = None

    category_page: Union[CategoryPage, None] = None
    product_page: Union[ProductPage, None] = None

    # SEÇİCİLER
    page_load_element = ()  # [seçici türü, seçici]
    cookie_element = ()  # [seçici türü, seçici]
    category_link_locator = ()
    category_name_locator = ()

    # JAVASCRIPTS
    remove_element_script = "arguments[0].remove()"
    scroll_to_element_script = "arguments[0].scrollIntoView({block: 'center'});"
    scroll_to_bottom_script = "window.scrollTo(0, document.body.scrollHeight);"

    def __new__(cls) -> Self:

        if not cls.browser or not cls.driver:
            cls.browser = Browser()
            cls.driver = cls.browser.disable_logging() \
                .increase_performance() \
                .set_window_size() \
                .run() \
                .maximize_window() \
                .get()
            cls.wait = WebDriverWait(cls.driver, 10)

        return super().__new__(cls)

    def __init__(self):
        # tarih bilgileri Türkçe olması için çeviri yap
        locale.setlocale(locale.LC_TIME, "tr_TR")

        # veritabanı çalıştırılır
        self.db = DB(self.db_path)

    def is_page_load(self):
        """Gelen locator bilgisine göre sayfanın yüklenmesini bekler"""
        self.wait.until(ec.visibility_of_element_located(
            self.page_load_element
        ))

    def accept_cookie(self):
        """Sitede çerezleri kabul edecek metot"""
        try:
            self.wait.until(ec.element_to_be_clickable(
                self.cookie_element
            )).click()
            self.logger.info("Çerezler kabul edildi")
        except TimeoutException:
            self.logger.info("Çerez kabul etme paneli gelmedi")

    def go(self):
        self.driver.get(self.URL)
        self.is_page_load()
        self.accept_cookie()

        self.logger.info(f"'{self.URL}' adresine gidildi.")

    def remove_elements(self, locator: Tuple[str, str], element: Union[WebElement, None] = None):
        # eğer gönderilen bir web elemanı değilse seç
        select_from = self.driver if element is None else element
        try:
            els = select_from.find_elements(*locator)
        except:
            els = []

        for el in els:
            self.driver.execute_script(self.remove_element_script, el)

    def scroll_to_element(self, el: Union[WebElement, Tuple[str, str]]):
        # eğer gönderilen bir web elemanı değilse seç
        if not isinstance(el, WebElement):
            el = self.driver.find_element(*el)

    def scroll_to_bottom(self):
        self.driver.execute_script(self.scroll_to_bottom_script)

    def get_next_date(self, excludes: List, locator: Tuple[str, str]) -> Dict:
        els = self.driver.find_elements(*locator)
        dates = Date.get_dates(els, excludes)
        return Date.next_date(dates)

    def create_date_directory(self, date: Union[Dict, None] = None):
        date = date["object"] if date else date
        date_str = Date.to_str(date)
        return File.mk_dir(self.IMAGES_DIR, date=date_str)

    def get_product_elements(self, locator: Tuple[str, str]):
        return self.driver.find_elements(*locator)

    def update_excludes(self, txt: str, key: str, excludes: List):
        excludes.append(txt)
        self.db.set(key, excludes)

    def save_last_scrape_date(self):
        today_str = Date.to_str()
        self.db.set("last_scrape_date", today_str)
        self.logger.info(f"Kontrol tarihi kaydedildi: {today_str}")

    def is_it_time_to_scrape(self, threshold_days: int = 7) -> bool:
        """Gönderilen son kazıma tarihi baz alınarak, istenen sürenin geçip
        geçmediğini kontrol ederek "kazıma zamanı geldi mi" karar verir.

        Args:
            threshold_days (int, optional): Geçmesi gereken süre. Defaults to 7.

        Returns:
            bool: "Kazıma zamanı geldi mi" sonucu
        """
        last_date = self.db.get("last_scrape_date")

        # son tarih yoksa daha önce bilgiler alınmamıştır.
        # Bu durumda bilgiler alınabilir.
        if last_date is None:
            return True

        difference = Date.day_difference(last_date)

        # Farkı gün cinsine dönüştür ve threshold_days ile karşılaştır
        if difference >= threshold_days:
            return True

        self.logger.info(
            f"Son kontrolden '{difference}' gün geçtiği için iptal edildi.")
        return False

    def get_products(self) -> Dict:
        # kontrol tarihi geldi mi
        if not self.is_it_time_to_scrape():
            return {}

        products = self.get_discount_product()

        if products:
            self.save_last_scrape_date()

        self.driver.close()
        return products

    def get_discount_product(self) -> Dict:
        self.go()

        # Önce kategoriler alınır.
        categories = self.get_categories()

        # kategorilerden indirimli ürünler alınır
        product_list = self.fetch_category_products(
            self.category_page,
            categories
        )

        self.IMAGES_DIR = self.create_date_directory()
        return self.fetch_products(self.product_page, product_list)

    def get_categories(self) -> List:
        category_elems = self.driver.find_elements(*self.category_link_locator)

        categories = []

        for el in category_elems:
            categories.append({
                "link": el.get_attribute("href"),
                "title": el.find_element(*self.category_name_locator).text.strip()
            })

        self.logger.info(f"{len(categories)} adet ürün kategorisi alındı.")
        return categories

    def merge_products(self, data: List) -> List:
        """Gelen kategori listesindeki, her kategoride bulunan ürünleri
        tek bir liste haline getirir

        Args:
            data (List): İçinde ürünlerin de bulunduğu kategorilerden oluşan liste

        Returns:
            List: Ürün listesi
        """
        new_data = []
        for category in data:
            for product in category["products"]:
                product["category"] = category["title"]
                new_data.append(product)

        return new_data

    def fetch_category_products(
        self,
        category_page: CategoryPage,
        categories: List[Dict]
    ) -> List:

        # category: {"link": "...", "title": "..."}
        data = []
        for category in categories:
            page: CategoryPage = category_page(self, **category)
            page.go()
            # ilgili kategori ürün bilgileri alınırken bir hata oluşursa geç
            try:
                page.scrape_products()
            except:
                continue
            data.append(page.data)

        return self.merge_products(data)

    def fetch_products(self, product_page: ProductPage, products: List) -> Dict:
        data = {}  # 'code': product

        # Ürünler gezilir
        # product: {
        #   'category': '...',
        #   'sub_category': '...',
        #   'name': '...',
        #   'link': '...',
        #   'code': '...'
        # }
        for product in products:
            # eğer aynı ürün varsa geç
            if product["code"] in data.keys():
                continue

            try:
                page = product_page(self, **product)
                page.go()
                result = page.get_product()
                data[result["code"]] = result
            except Exception as e:
                self.logger.error("Ürün Bilgileri alınırken hata oldu: "
                                  f"{product}")
                self.logger.error(f"--- Hata: {e}")

        return data



from typing import Dict, Tuple, Union
from ..date import Date
from ..filesytem import File
from selenium.webdriver.remote.webelement import WebElement
from selenium.common.exceptions import NoSuchElementException


class Product:

    _logger = None

    brand_locator = ()
    name_locator = ()
    description_locator = ()
    old_price_locator = ()
    price_locator = ()
    discount_rate_locator = ()
    has_image = True
    image_locator = ()
    image_is_download = False

    def __init__(
        self,
        element: WebElement,
        image_path: Union[str, None] = None,
        **kwargs
    ) -> None:
        """Web sitesindeki ürün elemanı üzerinde işlemler yapar.
        Bilgiler toplar ve görseli kaydeder.

        Args:
            element (WebElement): Selenium Web Element nesnesi
            image_path (Union[str, None], optional): Görselin kaydedileceği dizin. Defaults to None.
        """
        self._element = element
        self._image_path = image_path

        for key in kwargs.keys():
            setattr(self, key, kwargs.get(key))

        if self.brand_locator and not hasattr(self, "brand"):
            self.brand = self.get_brand()

        if self.name_locator and not hasattr(self, "name"):
            self.name = self.get_name()

        if self.description_locator and not hasattr(self, "description"):
            self.description = self.get_description()

        if self.old_price_locator and not hasattr(self, "old_price"):
            self.old_price = self.get_old_price()

        if self.price_locator and not hasattr(self, "price"):
            self.price = self.get_price()

        if self.discount_rate_locator and not hasattr(self, "discount_rate"):
            self.discount_rate = self.get_discount_rate()

        if self.has_image and not hasattr(self, "image"):
            self.image = self.get_image()

    def get_brand(self) -> Union[str, None]:
        try:
            el = self._element.find_element(*self.brand_locator)
            text = el.text.strip()
            self._logger.info(f"--- Marka: {text}")
            return text
        except NoSuchElementException:
            return None

    def get_name(self) -> Union[str, None]:
        try:
            el = self._element.find_element(*self.name_locator)
            text = el.text.strip()
            self._logger.info(f"--- Ürün Adı: {text}")
            return text
        except NoSuchElementException:
            return None

    def get_description(self) -> Union[str, None]:
        try:
            el = self._element.find_element(*self.description_locator)
            text = el.text.strip()
            self._logger.info(f"--- Açıklama: {text}")
            return text
        except NoSuchElementException:
            return None

    def get_old_price(self) -> Union[str, None]:
        try:
            el = self._element.find_element(*self.old_price_locator)
            text = el.text.strip()
            self._logger.info(f"--- Eski Fiyat: {text}")
            return text
        except NoSuchElementException:
            return None

    def get_price(self) -> Union[str, None]:
        try:
            el = self._element.find_element(*self.price_locator)
            text = el.text.strip()
            self._logger.info(f"--- Fiyat: {text}")
            return text
        except NoSuchElementException:
            return None

    def get_discount_rate(self) -> Union[str, None]:
        try:
            el = self._element.find_element(*self.discount_rate_locator)
            text = el.text.strip()
            self._logger.info(f"--- İndirim Oranı: {text}")
            return text
        except NoSuchElementException:
            return None

    def _image_name(self) -> str:
        timestamp = Date.timestamp()
        filename = f"{self.name}-{timestamp}"
        filename = File.clean_filename(filename)
        return f"{self._image_path}/{filename}.png"

    def get_image(self) -> str:
        # görsel seçici varsa yeni eleman belirle
        el = self._element
        if self.image_locator:
            el = self._element.find_element(*self.image_locator)

        # eğer indirme ise indir
        if self.image_is_download:
            src = el.get_attribute("src")
            image = File.download(src, self._image_path)
            self._logger.info(f"--- Görsel: {image}")
            return image

        image = self._image_name()
        el.screenshot(image)
        self._logger.info(f"--- Görsel: {image}")
        return image

    def has_attribute(self, locator: Tuple[str, str]) -> bool:
        try:
            el = self._element.find_element(*locator)
            if el:
                return True
        except NoSuchElementException:
            pass

        return False

    def get_price_text(self, html_text: str) -> str:
        return html_text.replace("₺", "").replace("\n", "")

    def price_to_float(self, price_text: str) -> float:
        price_text = price_text.replace(".", "").replace(",", ".")
        return float(price_text)

    def get_data(self) -> Dict:
        data = {}

        for attr, value in vars(self).items():
            if attr.startswith('_') or attr.startswith('__'):
                continue
            data[attr] = value

        return data



from typing import Dict
from urllib.parse import urlparse, parse_qs


class ParsedURL:
    def __init__(self, url):
        self.url = url
        self.parsed_url = urlparse(url)
        self.scheme = self.parsed_url.scheme
        self.netloc = self.parsed_url.netloc
        self.path = self.parsed_url.path
        self.params = self.parsed_url.params
        self.query = self.parsed_url.query
        self.fragment = self.parsed_url.fragment
        self.query_params = parse_qs(self.query)

    def get_scheme(self) -> str:
        return self.scheme

    def get_netloc(self) -> str:
        return self.netloc

    def get_path(self) -> str:
        return self.path

    def get_params(self) -> str:
        return self.params

    def get_query_text(self) -> str:
        return self.query

    def get_query(self) -> Dict:
        return self.query_params

    def get_fragment(self) -> str:
        return self.fragment

    def get_query_param(self, param_name) -> str:
        return self.get_query().get(param_name, None)

    def get_home(self) -> str:
        return f"{self.get_scheme()}://{self.get_netloc()}"

    def make_adress(self, suffix: str) -> str:
        if suffix[0] == "/":
            suffix = suffix[1:]
        return f"{self.get_home()}/{suffix}"



import logging
import requests
from typing import List, Union
from pdf2image import convert_from_path
from file_system.file import File as FileCore


logger = logging.getLogger("MarketBaseFile")


class File:

    @staticmethod
    def mk_dir(base_path: str, **kwargs) -> str:
        path = base_path.format(**kwargs)
        if not FileCore.exists(path):
            FileCore.create_directory(path)
            logger.info(f"'{path}' dizini oluşturuldu.")

        return path

    @staticmethod
    def clean_filename(filename: str):
        # İstenmeyen karakterlerin bulunduğu bir karakter kümesi oluşturun
        invalid_chars = '<>:"/\\|?*'

        # İstenmeyen karakterleri temizleyin
        chars = [c for c in filename if c not in invalid_chars]
        cleaned_filename = ''.join(chars)
        logger.info("Dosya adı temizlendi. "
                    f"'{filename}' -> '{cleaned_filename}'")

        return cleaned_filename

    @staticmethod
    def download(src: str, dest: str) -> Union[str, None]:
        """
        İnternetten bir kaynağın indirilmesini ve hedefe kaydedilmesini sağlar

        Args:
            src (str): kaynak adresi
            dest (str): yerel hedef yolu

        Returns:
            str: _description_
        """
        # dosya adı belirlenir
        filename = FileCore.get_filename(src)

        try:
            response = requests.get(src)
            if response.status_code == 200:
                file_path = f"{dest}/{filename}"
                with open(f"{dest}/{filename}", "wb") as file:
                    file.write(response.content)

                return file_path
            else:
                logger.error(f"----- {src} adresinden dosya indirilemedi.")
                return None
        except Exception as e:
            logger.error(f"----- Dosya indirilirken hata oluştu: {e}")
            return None

    @staticmethod
    def pdf_to_images(pdf_path: str, dest: str) -> List:
        image_paths = []

        # PDF dosyasını görüntü dosyalarına dönüştür
        images = convert_from_path(pdf_path)

        # dosya adını al
        filename = FileCore.get_filename(pdf_path, without_extentsion=True)

        for i, image in enumerate(images, start=1):
            # görüntü dosyasının adını oluştur
            image_path = f"{dest}/{filename}_page_{i}.jpg"
            image.save(image_path)
            image_paths.append(image_path)
            logger.info(f"'{image_path}' görseli oluşturuldu.")

        return image_paths



import logging
from datetime import datetime
from typing import Dict, List, Union
from selenium.webdriver.remote.webelement import WebElement


logger = logging.getLogger("MarketBaseDate")


class Date:

    @staticmethod
    def to_str(
        date: Union[datetime, None] = None,
        date_format: str = "%Y-%m-%d"
    ) -> str:
        if date is None:
            date = datetime.today()

        return date.strftime(date_format)

    @staticmethod
    def timestamp(date: Union[datetime, None] = None) -> int:
        if date:
            return date.timestamp()

        return datetime.now().timestamp()

    @classmethod
    def parse(cls, txt: str, _format: str = "%d %B %A") -> datetime:
        """Gelen tarih string verisini datetime nesnesine döndürür.
        Yılı hesaplar.

        Args:
            txt (str): tarih metin değeri
            _format (str): metinden çıkarılacak tarih formatı

        Returns:
            datetime: oluşturulan tarih nesnesi
        """
        if "-" in txt:
            return cls._parse_range(txt)

        return cls._parse_single(txt, _format)

    @classmethod
    def _parse_range(cls, txt: str, _format: str = "%d %B"):
        finish = txt.split('-')[1].strip()
        return cls._parse_single(finish, _format)

    @classmethod
    def _parse_single(cls, txt: str, _format: str) -> datetime:
        # Ay adı alınır
        month_name = cls.get_month_name(txt, _format)

        # Bulunduğumuz yıl alınır
        year = datetime.today().year

        # Bulunduğumuz ay Aralık ise ve
        # gelen tarih Ocak ayı ise yılı 1 arttır
        if datetime.today().month == 12 and month_name == "Ocak":
            year += 1

        # Belirlenen yıla göre tarih nesnesi oluştur
        _format += " %Y"
        return datetime.strptime(f"{txt} {year}", _format)

    @staticmethod
    def get_month_name(txt: str, _format: str):
        """Gelen tarih metni ve formatına göre ayın adını alır.
        Bunun için formattaki %B ile ay adının konumunu bulur 
        ve metinden döndürür.

        Args:
            txt (str): Tarih metni
            _format (str): Tarih formatı

        Returns:
            str: ay adı
        """
        format_parts = _format.split()
        i = format_parts.index("%B")
        return txt.split()[i]

    @classmethod
    def get_dates(
        cls,
        els: List[WebElement],
        excludes: Union[List, None]
    ) -> List:
        dates = []

        for el in els:
            # eğer tarih alınamadı ise geç
            try:
                date = cls.get_date(el, excludes)
                if date:
                    dates.append(date)
            except:
                continue

        logger.info(f"{len(dates)} adet tarih bilgisi alındı")

        return dates

    @classmethod
    def get_date(
        cls,
        el: WebElement,
        excludes: Union[List, None]
    ) -> Union[Dict, None]:
        if excludes and el.text in excludes:
            return

        return cls.date_dict(el)

    @classmethod
    def date_dict(cls, el: WebElement) -> Dict:
        return {
            "element": el,
            "text": el.text,
            "object": cls.parse(el.text)
        }

    @staticmethod
    def next_date(dates: List[Dict]) -> Dict:
        # Bugünün tarih ve saat bilgisini al
        today = datetime.today()

        # Saat ve dakikayı sıfırla
        today = today.replace(hour=0, minute=0, second=0, microsecond=0)

        return next((d for d in dates if d["object"] >= today), None)

    @staticmethod
    def day_difference(
        start_date: Union[str, datetime],
        finish_date: Union[str, datetime, None] = None
    ) -> int:
        """İki tarih arasındaki gün farkını bulur

        Args:
            start_date (str): son kazıma tarihi. Format: "%Y-%m-%d"
            threshold_days (int, optional): Geçmesi gereken süre. Defaults to 7.

        Returns:
            bool: "Kazıma zamanı geldi mi" sonucu
        """

        # String tarihini datetime nesnesine dönüştür
        if isinstance(start_date, str):
            start_date = datetime.strptime(start_date, "%Y-%m-%d")

        if finish_date is None:
            finish_date = datetime.today()
        elif isinstance(finish_date, str):
            finish_date = datetime.strptime(finish_date, "%Y-%m-%d")

        # Tarih farkını hesapla
        difference = finish_date - start_date

        # Farkı gün cinsine dönüştür ve threshold_days ile karşılaştır
        return difference.days



from typing import Dict, Union
from .product import Product
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.chrome.webdriver import WebDriver
from selenium.webdriver.remote.webelement import WebElement


class ProductPage:

    driver: Union[WebDriver, None] = None
    wait: Union[WebDriverWait, None] = None
    _logger = None
    product: Union[Product, None] = None

    product_locator = ()

    def __init__(self, main, **kwargs) -> None:
        self.main = main
        self.driver = self.main.driver
        self.wait = WebDriverWait(self.driver, 10)
        self.kwargs = kwargs

    def go(self):
        self.driver.get(self.kwargs["link"])
        self.main.is_page_load()
        info = self.kwargs.get("name", self.kwargs.get("link"))
        self._logger.info(f"'{info}' ürünü işlemleri başladı.")

    def pre_process(self, el: WebElement):
        """
        Bu metot extend edilen sınıflarda genişletilebilir. Belirli 
        script'lerle sayfadaki bazı değişiklikler yapılabilir. Örneğin; 
        bazı elemanlar silinebilir, eleman boyutları düzeltilebilir.
        """
        pass

    def get_product(self) -> Dict:
        by_type, selector = self.product_locator
        selector = selector.format(**self.kwargs)
        el = self.driver.find_element(by_type, selector)

        self.pre_process(el)

        product = self.product(el, self.main.IMAGES_DIR, **self.kwargs)

        return product.get_data()



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    description = fh.read()

setup(
    name="emo_market_base",
    version="v0.0.3",
    author="Eren Mustafa Özdal",
    author_email="eren.060737@gmail.com",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    description="Marketlerden ürünleri kazıma işlemleri için temel pakettir",
    long_description=description,
    long_description_content_type="text/markdown",
    url="https://github.com/erenmustafaozdal/market-base",
    license='MIT',
    python_requires='>=3.11',
    install_requires=[
        "selenium_web_browser",
        "emo-cache-db",
        "emo-file-system",
        "pillow",
        "pdf2image",
    ],
)



import logging
from typing import Any, Union
from cache_db.cache_db import CacheDB


class DB:

    def __init__(self, path: str) -> None:
        self.db = CacheDB(path)
        self.db.load_cache()

        self.logger = logging.getLogger("MarketBaseDB")

    def get(self, key: str, default: Union[Any, None] = None) -> Any:
        result = self.db.get(key, default)
        self.logger.info(f"Veri alındı. '{key}': '{result}'")
        return result

    def set(self, key: str, value, with_save: bool = True):
        self.db.cache[key] = value
        self.logger.info(f"Veri eklendi. '{key}': '{value}'")

        if with_save:
            self.db.save_cache()
            self.logger.info(f"Dosya kaydedildi: {self.db.file_path}")



from time import sleep
from typing import Dict, List, Union
from .list_product import ListProduct
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.chrome.webdriver import WebDriver
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.support import expected_conditions as ec


class CategoryPage:

    driver: Union[WebDriver, None] = None
    wait: Union[WebDriverWait, None] = None
    logger = None
    list_product: Union[ListProduct, None] = None

    with_loader: bool = False
    product_locator = ()

    def __init__(self, main, **kwargs) -> None:
        self.main = main
        self.driver = self.main.driver
        self.wait = WebDriverWait(self.driver, 10)

        self.link = kwargs.get("link")
        self.title = kwargs.get("title")
        self.data = {
            "link": self.link,
            "title": self.title,
            "products": []
        }

    def go(self):
        self.driver.get(self.link)
        self.main.is_page_load()
        self.logger.info(f"'{self.title}' kategorisi işlemleri başladı.")

    def scrape_products(self):
        self.data["products"] += self.fetch_products()

        self.logger.info(f"'{self.title}' kategorisi tamamlandı.")
        self.logger.info("--- İndirimli ürün sayısı: "
                         f"{len(self.data['products'])}")

    def get_products(self, el: Union[WebElement, None] = None) -> List:
        # eğer loader varsa tüm ürünlerin sayfaya
        # yüklenmesi için load işlemi yapılır
        product_count = 0
        while True:
            # sayfadaki ürünler alınır
            products = self.wait.until(ec.visibility_of_all_elements_located(
                self.product_locator
            ))

            # eğer loader yoksa döngüden çık
            if not self.with_loader:
                break

            # eğer sayfadaki ürün sayısı bir eki
            # döngüdeki ürün sayısı ile aynı ise
            # bu yeni ürün gelmedi demektir: çık
            if len(products) == product_count:
                break

            product_count = len(products)

            # sayfaya yeni eleman yükleme işlemi: kaydırm, button vs.
            self.load()
            sleep(0.5)

        return products

    def load(self):
        self.main.scroll_to_bottom()

    def fetch_products(self) -> List:
        els = self.get_products()

        data = []
        for el in els:
            result = self.fetch_product(el)
            if result:
                data.append(result)

        return data

    def fetch_product(self, el: WebElement) -> Union[Dict, None]:
        p_list = self.list_product(el)

        if p_list.is_discount():
            return {
                "name": p_list.name,
                "link": p_list.link,
                "code": p_list.code,
            }
